@@ -27,6 +27,9 @@
 
 typedef void (* MetaAnimationFinishedFunc) (gpointer data);
 
+/* XXX namespace me */
+void compute_window_rect (MetaWindow *window, MetaRectangle *rect);
+
 MetaCompositor* meta_compositor_new           (MetaDisplay       *display);
 void            meta_compositor_unref         (MetaCompositor    *compositor);
 void            meta_compositor_process_event (MetaCompositor    *compositor,
@@ -45,6 +48,7 @@ void meta_compositor_manage_screen   (MetaCompositor *compositor,
 void meta_compositor_unmanage_screen (MetaCompositor *compositor,
                                       MetaScreen     *screen);
 
+#if 0
 void meta_compositor_minimize (MetaCompositor            *compositor,
 			       MetaWindow                *window,
 			       int                        x,
@@ -53,6 +57,7 @@ void meta_compositor_minimize (MetaCompositor            *compositor,
 			       int                        height,
 			       MetaAnimationFinishedFunc  finished_cb,
 			       gpointer                   finished_data);
+#endif
 
 void
 meta_compositor_set_updates (MetaCompositor *compositor,
