@@ -22,6 +22,7 @@
 #ifndef META_THEME_H
 #define META_THEME_H
 
+#include "boxes.h"
 #include "gradient.h"
 #include "common.h"
 #include <gtk/gtkrc.h>
@@ -31,8 +32,10 @@ typedef struct _MetaFrameStyleSet MetaFrameStyleSet;
 typedef struct _MetaDrawOp MetaDrawOp;
 typedef struct _MetaDrawOpList MetaDrawOpList;
 typedef struct _MetaGradientSpec MetaGradientSpec;
+typedef struct _MetaAlphaGradientSpec MetaAlphaGradientSpec; 
 typedef struct _MetaColorSpec MetaColorSpec;
 typedef struct _MetaFrameLayout MetaFrameLayout;
+typedef struct _MetaButtonSpace MetaButtonSpace;
 typedef struct _MetaFrameGeometry MetaFrameGeometry;
 typedef struct _MetaTheme MetaTheme;
 typedef struct _MetaPositionExprEnv MetaPositionExprEnv;
@@ -51,6 +54,12 @@ typedef enum
   META_THEME_ERROR_FAILED
 } MetaThemeError;
 
+typedef enum
+{
+  META_BUTTON_SIZING_ASPECT,
+  META_BUTTON_SIZING_FIXED,
+  META_BUTTON_SIZING_LAST
+} MetaButtonSizing;
 
 /* Parameters used to calculate the geometry of the frame */
 struct _MetaFrameLayout
@@ -73,6 +82,10 @@ struct _MetaFrameLayout
   int left_titlebar_edge;
   
   /* Size of buttons */
+  MetaButtonSizing button_sizing;
+
+  double button_aspect; /* height / width */
+  
   int button_width;
   int button_height;
 
@@ -84,8 +97,24 @@ struct _MetaFrameLayout
   
   /* Whether title text will be displayed */
   guint has_title : 1;
+
+  /* Round corners */
+  guint top_left_corner_rounded : 1;
+  guint top_right_corner_rounded : 1;
+  guint bottom_left_corner_rounded : 1;
+  guint bottom_right_corner_rounded : 1;
 };
 
+struct _MetaButtonSpace
+{
+  /* The reason for two different rectangles here is Fitts' law & maximized
+   * windows; see #97703 for more details.
+   *   visible   - The screen area where the button's image is drawn
+   *   clickable - The screen area where the button can be activated by clicking
+   */
+  GdkRectangle visible;
+  GdkRectangle clickable;
+};
 
 /* Calculated actual geometry of the frame */
 struct _MetaFrameGeometry
@@ -96,26 +125,53 @@ struct _MetaFrameGeometry
   int bottom_height;
 
   int width;
-  int height;
-  
-  GdkRectangle close_rect;
-  GdkRectangle max_rect;
-  GdkRectangle min_rect;
-  GdkRectangle menu_rect;
+  int height;  
+
   GdkRectangle title_rect;
 
   int left_titlebar_edge;
   int right_titlebar_edge;
   int top_titlebar_edge;
   int bottom_titlebar_edge;
+
+  /* used for a memset hack */
+#define ADDRESS_OF_BUTTON_RECTS(fgeom) (((char*)(fgeom)) + G_STRUCT_OFFSET (MetaFrameGeometry, close_rect))
+#define LENGTH_OF_BUTTON_RECTS (G_STRUCT_OFFSET (MetaFrameGeometry, right_right_background) + sizeof (GdkRectangle) - G_STRUCT_OFFSET (MetaFrameGeometry, close_rect))
+  
+  /* The button rects (if changed adjust memset hack) */
+  MetaButtonSpace close_rect;
+  MetaButtonSpace max_rect;
+  MetaButtonSpace min_rect;
+  MetaButtonSpace menu_rect;
+
+#define MAX_MIDDLE_BACKGROUNDS (MAX_BUTTONS_PER_CORNER - 2)
+  GdkRectangle left_left_background;
+  GdkRectangle left_middle_backgrounds[MAX_MIDDLE_BACKGROUNDS];
+  GdkRectangle left_right_background;
+  GdkRectangle right_left_background;
+  GdkRectangle right_middle_backgrounds[MAX_MIDDLE_BACKGROUNDS];
+  GdkRectangle right_right_background;
+  /* End of button rects (if changed adjust memset hack) */
+  
+  /* Round corners */
+  guint top_left_corner_rounded : 1;
+  guint top_right_corner_rounded : 1;
+  guint bottom_left_corner_rounded : 1;
+  guint bottom_right_corner_rounded : 1;
 };
 
+typedef enum
+{
+  META_IMAGE_FILL_SCALE, /* default, needs to be all-bits-zero for g_new0 */
+  META_IMAGE_FILL_TILE
+} MetaImageFillType;
 
 typedef enum
 {
   META_COLOR_SPEC_BASIC,
   META_COLOR_SPEC_GTK,
-  META_COLOR_SPEC_BLEND
+  META_COLOR_SPEC_BLEND,
+  META_COLOR_SPEC_SHADE
 } MetaColorSpecType;
 
 typedef enum
@@ -148,6 +204,10 @@ struct _MetaColorSpec
       MetaColorSpec *background;
       double alpha;
     } blend;
+    struct {
+      MetaColorSpec *base;
+      double factor;
+    } shade;
   } data;
 };
 
@@ -157,6 +217,13 @@ struct _MetaGradientSpec
   GSList *color_specs;
 };
 
+struct _MetaAlphaGradientSpec
+{
+  MetaGradientType type;
+  unsigned char *alphas;
+  int n_alphas;
+};
+
 struct _MetaDrawInfo
 {
   GdkPixbuf   *mini_icon;
@@ -244,7 +311,7 @@ struct _MetaDrawOp
     
     struct {
       MetaColorSpec *color_spec;
-      double alpha;
+      MetaAlphaGradientSpec *alpha_spec;
       char *x;
       char *y;
       char *width;
@@ -253,7 +320,7 @@ struct _MetaDrawOp
 
     struct {
       MetaGradientSpec *gradient_spec;
-      double alpha;
+      MetaAlphaGradientSpec *alpha_spec;
       char *x;
       char *y;
       char *width;
@@ -262,14 +329,17 @@ struct _MetaDrawOp
 
     struct {
       MetaColorSpec *colorize_spec;
+      MetaAlphaGradientSpec *alpha_spec;
       GdkPixbuf *pixbuf;
-      double alpha;
       char *x;
       char *y;
       char *width;
       char *height;
       guint32 colorize_cache_pixel;
       GdkPixbuf *colorize_cache_pixbuf;
+      MetaImageFillType fill_type;
+      unsigned int vertical_stripes : 1;
+      unsigned int horizontal_stripes : 1;
     } image;
     
     struct {
@@ -300,11 +370,12 @@ struct _MetaDrawOp
     } gtk_vline;
 
     struct {
-      double alpha;
+      MetaAlphaGradientSpec *alpha_spec;
       char *x;
       char *y;
       char *width;
       char *height;
+      MetaImageFillType fill_type;
     } icon;
 
     struct {
@@ -354,6 +425,13 @@ typedef enum
 
 typedef enum
 {
+  /* Ordered so that background is drawn first */
+  META_BUTTON_TYPE_LEFT_LEFT_BACKGROUND,
+  META_BUTTON_TYPE_LEFT_MIDDLE_BACKGROUND,
+  META_BUTTON_TYPE_LEFT_RIGHT_BACKGROUND,
+  META_BUTTON_TYPE_RIGHT_LEFT_BACKGROUND,
+  META_BUTTON_TYPE_RIGHT_MIDDLE_BACKGROUND,
+  META_BUTTON_TYPE_RIGHT_RIGHT_BACKGROUND,
   META_BUTTON_TYPE_CLOSE,
   META_BUTTON_TYPE_MAXIMIZE,
   META_BUTTON_TYPE_MINIMIZE,
@@ -500,10 +578,7 @@ struct _MetaTheme
 
 struct _MetaPositionExprEnv
 {
-  int x;
-  int y;
-  int width;
-  int height;
+  MetaRectangle rect;
   /* size of an object being drawn, if it has a natural size */
   int object_width;
   int object_height;
@@ -533,12 +608,13 @@ void             meta_frame_layout_get_borders   (const MetaFrameLayout *layout,
                                                   int                   *bottom_height,
                                                   int                   *left_width,
                                                   int                   *right_width);
-void             meta_frame_layout_calc_geometry (const MetaFrameLayout *layout,
-                                                  int                    text_height,
-                                                  MetaFrameFlags         flags,
-                                                  int                    client_width,
-                                                  int                    client_height,
-                                                  MetaFrameGeometry     *fgeom);
+void             meta_frame_layout_calc_geometry (const MetaFrameLayout  *layout,
+                                                  int                     text_height,
+                                                  MetaFrameFlags          flags,
+                                                  int                     client_width,
+                                                  int                     client_height,
+                                                  const MetaButtonLayout *button_layout,
+                                                  MetaFrameGeometry      *fgeom);
 
 gboolean         meta_frame_layout_validate      (const MetaFrameLayout *layout,
                                                   GError               **error);
@@ -572,11 +648,7 @@ void           meta_draw_op_draw (const MetaDrawOp    *op,
                                   const GdkRectangle  *clip,
                                   const MetaDrawInfo  *info,
                                   /* logical region being drawn */
-                                  int                  x,
-                                  int                  y,
-                                  int                  width,
-                                  int                  height);
-
+                                  MetaRectangle        logical_region);
 
 MetaDrawOpList* meta_draw_op_list_new   (int                   n_preallocs);
 void            meta_draw_op_list_ref   (MetaDrawOpList       *op_list);
@@ -585,11 +657,8 @@ void            meta_draw_op_list_draw  (const MetaDrawOpList *op_list,
                                          GtkWidget            *widget,
                                          GdkDrawable          *drawable,
                                          const GdkRectangle   *clip,
-                                         const MetaDrawInfo  *info,
-                                         int                   x,
-                                         int                   y,
-                                         int                   width,
-                                         int                   height);
+                                         const MetaDrawInfo   *info,
+                                         MetaRectangle         rect);
 void           meta_draw_op_list_append (MetaDrawOpList       *op_list,
                                          MetaDrawOp           *op);
 gboolean       meta_draw_op_list_validate (MetaDrawOpList    *op_list,
@@ -606,6 +675,10 @@ GdkPixbuf*        meta_gradient_spec_render (const MetaGradientSpec *desc,
 gboolean          meta_gradient_spec_validate (MetaGradientSpec     *spec,
                                                GError              **error);
 
+MetaAlphaGradientSpec* meta_alpha_gradient_spec_new  (MetaGradientType       type,
+                                                      int                    n_alphas);
+void                   meta_alpha_gradient_spec_free (MetaAlphaGradientSpec *spec);
+
 
 MetaFrameStyle* meta_frame_style_new   (MetaFrameStyle *parent);
 void            meta_frame_style_ref   (MetaFrameStyle *style);
@@ -657,30 +730,28 @@ double meta_theme_get_title_scale (MetaTheme     *theme,
                                    MetaFrameType  type,
                                    MetaFrameFlags flags);
 
-void meta_theme_draw_frame (MetaTheme          *theme,
-                            GtkWidget          *widget,
-                            GdkDrawable        *drawable,
-                            const GdkRectangle *clip,
-                            int                 x_offset,
-                            int                 y_offset,
-                            MetaFrameType       type,
-                            MetaFrameFlags      flags,
-                            int                 client_width,
-                            int                 client_height,
-                            PangoLayout        *title_layout,
-                            int                 text_height,
-                            MetaButtonState     button_states[META_BUTTON_TYPE_LAST],
-                            GdkPixbuf          *mini_icon,
-                            GdkPixbuf          *icon);
+void meta_theme_draw_frame (MetaTheme              *theme,
+                            GtkWidget              *widget,
+                            GdkDrawable            *drawable,
+                            const GdkRectangle     *clip,
+                            int                     x_offset,
+                            int                     y_offset,
+                            MetaFrameType           type,
+                            MetaFrameFlags          flags,
+                            int                     client_width,
+                            int                     client_height,
+                            PangoLayout            *title_layout,
+                            int                     text_height,
+                            const MetaButtonLayout *button_layout,
+                            MetaButtonState         button_states[META_BUTTON_TYPE_LAST],
+                            GdkPixbuf              *mini_icon,
+                            GdkPixbuf              *icon);
 
 void meta_theme_draw_menu_icon (MetaTheme          *theme,
                                 GtkWidget          *widget,
                                 GdkDrawable        *drawable,
                                 const GdkRectangle *clip,
-                                int                 x_offset,
-                                int                 y_offset,
-                                int                 width,
-                                int                 height,
+                                MetaRectangle       offset_rect,
                                 MetaMenuIconType    type);
      
 void meta_theme_get_frame_borders (MetaTheme         *theme,
@@ -691,14 +762,14 @@ void meta_theme_get_frame_borders (MetaTheme         *theme,
                                    int               *bottom_height,
                                    int               *left_width,
                                    int               *right_width);
-void meta_theme_calc_geometry     (MetaTheme         *theme,
-                                   MetaFrameType      type,
-                                   int                text_height,
-                                   MetaFrameFlags     flags,
-                                   int                client_width,
-                                   int                client_height,
-                                   MetaFrameGeometry *fgeom);
-
+void meta_theme_calc_geometry (MetaTheme              *theme,
+                               MetaFrameType           type,
+                               int                     text_height,
+                               MetaFrameFlags          flags,
+                               int                     client_width,
+                               int                     client_height,
+                               const MetaButtonLayout *button_layout,
+                               MetaFrameGeometry      *fgeom);
                                    
 MetaFrameLayout*   meta_theme_lookup_layout       (MetaTheme         *theme,
                                                    const char        *name);
@@ -742,8 +813,9 @@ char*    meta_theme_replace_constants     (MetaTheme   *theme,
 /* random stuff */
 
 PangoFontDescription* meta_gtk_widget_get_font_desc        (GtkWidget            *widget,
-                                                            double                scale);
-int                   meta_pango_font_desc_get_text_height (PangoFontDescription *font_desc,
+                                                            double                scale,
+							    const PangoFontDescription *override);
+int                   meta_pango_font_desc_get_text_height (const PangoFontDescription *font_desc,
                                                             PangoContext         *context);
 
 
@@ -774,5 +846,8 @@ GtkShadowType         meta_gtk_shadow_from_string      (const char            *s
 const char*           meta_gtk_shadow_to_string        (GtkShadowType          shadow);
 GtkArrowType          meta_gtk_arrow_from_string       (const char            *str);
 const char*           meta_gtk_arrow_to_string         (GtkArrowType           arrow);
+MetaImageFillType     meta_image_fill_type_from_string (const char            *str);
+const char*           meta_image_fill_type_to_string   (MetaImageFillType      fill_type);
+
 
 #endif
